@@ -1,6 +1,6 @@
 ;;; mu4e-mime-parts.el --- Dealing with MIME-parts & URLs -*- lexical-binding: t -*-
 
-;; Copyright (C) 2023 Dirk-Jan C. Binnema
+;; Copyright (C) 2023-2024 Dirk-Jan C. Binnema
 
 ;; Author: Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 ;; Maintainer: Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
@@ -24,19 +24,15 @@
 
 ;; Implements functions and variables for dealing with MIME-parts and URLs.
 
-
 ;;; TODO:
 ;; [~] mime part candidate sorting -> is his even possible generally?
 ;; [ ] URL support
 
 ;;; Code:
-
-
 (require 'mu4e-vars)
 (require 'mu4e-folders)
 (require 'gnus-art)
-
-
+(require 'crm)
 
 (defcustom mu4e-view-open-program
   (pcase system-type
@@ -54,7 +50,17 @@ specified a function as viewer."
   :type '(choice string function)
   :group 'mu4e-view)
 
-
+(defcustom mu4e-uniquify-save-file-name-function 'mu4e--uniquify-file-name
+  "Function to create a unique, not-yet-existing file name.
+
+Takes one parameter, a file-name path, and returns a file-name
+path that does not yet exist. This can be the same, or some
+variation.
+
+See `mu4e--uniquify-file-name' for an example."
+  :type 'function
+  :group 'mu4e-view)
+
 ;; remember the mime-handles, so we can clean them up when
 ;; we quit this buffer.
 (defvar-local mu4e~gnus-article-mime-handles nil)
@@ -66,11 +72,9 @@ specified a function as viewer."
     (mm-destroy-parts mu4e~gnus-article-mime-handles)
     (setq mu4e~gnus-article-mime-handles nil)))
 
-
 ;;; MIME-parts
 (defvar-local mu4e--view-mime-parts nil
   "Cached MIME parts for this message.")
-
 
 (defun mu4e-view-mime-parts()
   "Get the list of MIME parts for this message.
@@ -97,11 +101,14 @@ There are some internal fields as well, e.g. ; subject to change:
          (save-excursion
            (goto-char (point-min))
            (while (not (eobp))
-             (when-let ((part (get-text-property (point) 'gnus-data))
+             (when-let* ((part (get-text-property (point) 'gnus-data))
                         (index (get-text-property (point) 'gnus-part)))
                (when (and part (numberp index) (not (member index indices)))
                  (let* ((disp (mm-handle-disposition part))
                         (fname (mm-handle-filename part))
+                        (fname (and fname ;; massage
+                                    (gnus-map-function mm-file-name-rewrite-functions
+                                                       (file-name-nondirectory fname))))
                         (mime-type (mm-handle-media-type part))
                         (info
                          `(:part-index  ,index
@@ -135,8 +142,9 @@ There are some internal fields as well, e.g. ; subject to change:
 
 ;; https://emacs.stackexchange.com/questions/74547/completing-read-search-also-in-annotationsxc
 
-(defun mu4e--uniqify-file-name (fname)
-  "Return a non-yet-existing filename based on FNAME.
+(defun mu4e--uniquify-file-name (fname)
+  "Return a not-yet-existing filename based on FNAME.
+
 If FNAME does not yet exist, return it unchanged.
 Otherwise, return a file with a unique number appended to the base-name."
   (let ((num 1) (orig-name fname))
@@ -176,8 +184,8 @@ Otherwise, return a file with a unique number appended to the base-name."
   :keymap mu4e-view-completion-minor-mode-map)
 
 (defun mu4e--part-annotation (candidate part type longest-filename)
-  "Calculate the annotation candidates as per
-`:annotation-function' (see `completion-extra-properties')
+  "Calculate the annotation candidates as per annotation.
+I.e., `:annotation-function' (see `completion-extra-properties')
 
 CANDIDATE is the value to annotate.
 
@@ -227,7 +235,6 @@ information' is used for alignment."
         (format "%s" (concat "-> " target))))
       (_ (mu4e-error "Unsupported annotation type %s" type)))))
 
-
 (defvar helm-comp-read-use-marked)
 (defun mu4e--completing-read-real (prompt candidates multi)
   "Call the appropriate completion-read function.
@@ -282,21 +289,36 @@ Optionally,
 
 (defun mu4e-view-save-attachments (&optional ask-dir)
   "Save files from the current view buffer.
-This applies to all MIME-parts that are \"attachment-like\" (have a filename),
-regardless of their disposition.
+
+This applies to all MIME-parts that are \"attachment-like\" (have
+a filename), regardless of their disposition.
 
 With ASK-DIR is non-nil, user can specify the target-directory; otherwise
-one is determined using `mu4e-attachment-dir'."
+one is determined using `mu4e-attachment-dir'.
+
+This command assumes unique filenames for the attachments, since
+that is how the underlying completion mechanism works. If there
+are duplicates, only one is recognized.
+
+Furthermore, file-names that match `crm-separator' (by default, a
+comma and some optional whitespace) are not supported (see
+`completing-read-multiple' for further details). Hence, when we
+detect that, the function bails out and advises to use
+`mu4e-view-mime-part-action' instead, which does support such
+files."
   (interactive "P")
   (let* ((parts (mu4e-view-mime-parts))
          (candidates  (seq-map
-                         (lambda (fpart)
-                           (cons ;; (filename . annotation)
-                            (plist-get fpart :filename)
-                            fpart))
-                         (seq-filter
-                          (lambda (part) (plist-get part :attachment-like))
-                          parts)))
+                       (lambda (fpart)
+                         (let ((fname (plist-get fpart :filename)))
+                           (when (and crm-separator (string-match-p crm-separator fname))
+                             (mu4e-warn (concat  "File(s) match `crm-separator'; "
+                                                 "use mu4e-view-mime-part-action instead")))
+                           ;; (filename . annotation)
+                           (cons fname fpart)))
+                       (seq-filter
+                        (lambda (part) (plist-get part :attachment-like))
+                        parts)))
          (candidates (or candidates
                          (mu4e-warn "No attachments for this message")))
          (files (mu4e--completing-read "Save file(s): " candidates
@@ -306,11 +328,13 @@ one is determined using `mu4e-attachment-dir'."
     ;; we have determined what files to save, and where.
     (seq-do (lambda (fname)
               (let* ((part (cdr (assoc fname candidates)))
-                     (path (mu4e--uniqify-file-name
+                     (path (funcall mu4e-uniquify-save-file-name-function
                             (mu4e-join-paths
                              (or custom-dir (plist-get part :target-dir))
-                             (plist-get part :filename)))))
-                (mm-save-part-to-file (plist-get part :handle) path)))
+                             (plist-get part :filename))))
+                     (handle (plist-get part :handle)))
+                (when handle ;; completion may fail, and then there's no handle.
+                  (mm-save-part-to-file handle path))))
             files)))
 
 (defvar mu4e-view-mime-part-actions
@@ -361,12 +385,11 @@ Each of the actions is a plist with keys
                        ;; - index: the index number of the mime part (default)
                        ;; - temp: the full path to the mime part in a
                        ;;         temporary file, which is deleted immediately
-                       ;;         after invoking handler
+                       ;;         after the handler returns
                        ;; - pipe:  the attachment is piped to some shell command
                        ;;          or as a string parameter to a function
 ).")
 
-
 (defun mu4e--view-mime-part-to-temp-file (handle)
   "Write MIME-part HANDLE to a temporary file and return the file name.
 The filename is deduced from the MIME-part's filename, or
@@ -406,7 +429,7 @@ If N is not specified, ask for it. For instance, '3 A o' opens
 the third MIME-part."
   ;; (interactive
   ;;  (list (read-number "Number of MIME-part: ")))
-  (interactive)
+  (interactive "P")
   (let* ((parts (mu4e-view-mime-parts))
          (candidates (seq-map
                       (lambda (part)
@@ -481,6 +504,5 @@ the third MIME-part."
     (display-buffer buf)))
 
 
-
 (provide 'mu4e-mime-parts)
 ;;; mu4e-mime-parts.el ends here
